unit DBTree;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, IB, IBDatabase, IBCustomDataSet, IBQuery, ComCtrls, StdCtrls, IBSQL;

type
  TMainForm = class(TForm)
    tvTree: TTreeView;
    qryTree: TIBQuery;
    dbInvent: TIBDatabase;
    taTree: TIBTransaction;
    Label1: TLabel;
    bInsertInRoot: TButton;
    IBSQL1: TIBSQL;
    edParent: TEdit;
    edId: TEdit;
    edName: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    bInsertInCurrent: TButton;
    bDeleteGroup: TButton;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure NodeViewExpanding(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
    procedure GetExpandingItems(var Node: TTreeNode);
    procedure tvTreeDeletion(Sender: TObject; Node: TTreeNode);
    procedure tvTreeChange(Sender: TObject; Node: TTreeNode);
    procedure tvTreeCollapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure bInsertInRootClick(Sender: TObject);
    procedure bDeleteGroupClick(Sender: TObject);
    procedure bInsertInCurrentClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  { ItemRec         .
      Node  TreeView     Node.Data.
     ItemRec     TREE.}
  PItemRec = ^ItemRec;
  ItemRec = record
    Parent: string;
    Id: string;
    Full_Id: string;
    Name: string;
    Legatees: integer;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
var
  CanBeExpanded: Boolean;
begin
  //   . , ,     .
  try
    dbInvent.Connected := true;
    MainForm.Caption := 'DBTree -- ' + dbInvent.DatabaseName;
  except
    on EIBInterBaseError do
    begin
      Beep;
      MainForm.Caption := 'DBTree -- CONNECTION FAILED: ' + dbInvent.DatabaseName;
    end;
  end;
  CanBeExpanded := false;
  //     ,     (nil).
  NodeViewExpanding(self, nil, CanBeExpanded);
end;

procedure TMainForm.NodeViewExpanding(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
begin
  GetExpandingItems(Node);
end;

procedure TMainForm.GetExpandingItems(var Node: TTreeNode);
var
  NewNode: TTreeNode;
  R: PItemRec;
begin
  //  qryTree    - Full_Id   .
	//     -   '',   - 
  //   ItemRec,    - Node.Data
  if Node = nil then
    qryTree.ParamByName('Parent').asString := ''
  else
    begin
    	Node.Selected := true;
			R := Node.Data;
      qryTree.ParamByName('Parent').asString := R.Full_Id;
      //   ,    
      //     
      Node.DeleteChildren;
    end;

  taTree.Active := true;
  qryTree.Open;
  qryTree.First;

	//          
  while not qryTree.Eof do
    begin
      NewNode := tvTree.Items.AddChildObject(
      tvTree.Selected,
      qryTree.FieldByName('Full_Id').asString + ' ' +
      qryTree.FieldByName('Name').asString, nil);
      //         
      New(R);
      NewNode.Data := R;
      R.Parent := qryTree.FieldByName('Parent').asString;
      R.Id := qryTree.FieldByName('Id').asString;
      R.Full_Id := qryTree.FieldByName('Full_Id').asString;
      R.Name := qryTree.FieldByName('Name').asString;
      R.Legatees := qryTree.FieldByName('Legatees').asInteger;
			//    > 0 ,    (+).
      NewNode.HasChildren := R.Legatees > 0;
      qryTree.Next;
    end;
  //    -   :-)
  qryTree.Close;
  taTree.Active := false;
end;

procedure TMainForm.tvTreeDeletion(Sender: TObject; Node: TTreeNode);
begin
	//      (,   ),
  //     ,     
  Dispose(PItemRec(Node.Data));
end;

procedure TMainForm.tvTreeChange(Sender: TObject; Node: TTreeNode);
//      .
var
	R: PItemRec;
begin
	R := Node.Data;
  Label1.Caption := R.Full_Id + ' ' + R.Name + ':  - ' + IntToStr(R.Legatees);
  R := Node.Data;
  edParent.Text := R.Parent;
  edId.Text := R.Id;
	edName.Text := R.Name;
end;

procedure TMainForm.tvTreeCollapsing(Sender: TObject; Node: TTreeNode;
var AllowCollapse: Boolean);
begin
  Node.Selected := true;
end;

procedure TMainForm.bInsertInRootClick(Sender: TObject);
//    "   "   
var
  NewNode: TTreeNode;
  R: PItemRec;
begin
	//     SQL -  (INSERT)
  // Parent = '', ID, Name -  . TEdit
	IBSQL1.SQL.Clear;
  IBSQL1.SQL.Add('insert into Tree (Parent, Id, Name)');
  IBSQL1.SQL.Add('values('''', ''' + edId.Text + ''', ''' + edName.Text + ''');' );
  taTree.Active := true;
	IBSQL1.ExecQuery;
  taTree.Commit;
  New(R);
  R.Parent := edParent.Text;
  R.Id := edId.Text;
  R.Full_Id := R.Parent + R.Id;
  R.Name := edName.Text;
  R.Legatees := 0;
  //  TreeView  -  nil
	newNode := tvTree.Items.AddChildObject(nil, edParent.Text + edId.Text + ' ' + edName.Text, R);
	NewNode.HasChildren := false;
  NewNode.Selected := true;
  tvTree.SetFocus;
end;

procedure TMainForm.bDeleteGroupClick(Sender: TObject);
//   TIBSQL
begin
	if MessageDlg(':   -   ?' ,
    mtWarning, [mbYes, mbNo], 1) = mrYes then
    begin
		  IBSQL1.SQL.Clear;
		  IBSQL1.SQL.Add('delete from Tree');
		  IBSQL1.SQL.Add('where Parent=''' + edParent.Text + ''' and  Id = ''' + edId.Text + ''';' );
		  taTree.Active := true;
			IBSQL1.ExecQuery;
		  taTree.Commit;
		  tvTree.Items.Delete(tvTree.Selected);
  	end;
  tvTree.SetFocus;
end;

procedure TMainForm.bInsertInCurrentClick(Sender: TObject);
//   ( -).    .
//   ,  Parent -   ,  Full_Id  .
var
  NewNode: TTreeNode;
  R: PItemRec;
  S_Id: string;
begin
	R := tvTree.Selected.Data;
	IBSQL1.SQL.Clear;
  IBSQL1.SQL.Add('insert into Tree (Parent, Id, Name)');
  IBSQL1.SQL.Add('values(''' + R.Full_Id + ''', ''' + edId.Text + ''', ''' + edName.Text + ''');' );
  taTree.Active := true;
	IBSQL1.ExecQuery;
  taTree.Commit;
  R.Legatees := R.Legatees + 1;
  tvTree.Selected.HasChildren := true;
  S_Id := R.Full_Id;
  New(R);
  R.Parent := S_Id;
  R.Id := edId.Text;
  R.Full_Id := R.Parent + R.Id;
  R.Name := edName.Text;
  R.Legatees := 0;
	//   :      (nil),
  //     (tvTree.Selected)
	newNode := tvTree.Items.AddChildObject(tvTree.Selected, R.Full_Id + ' ' + R.Name, R);
	NewNode.HasChildren := false;
  NewNode.Selected := true;
  tvTree.SetFocus;
end;

end.
